package cn.iocoder.yudao.module.order.controller.admin.order;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.OrderAssociationOfferInfoEvent;
import cn.iocoder.yudao.framework.common.util.spring.enums.UserTypeEnum;
import cn.iocoder.yudao.framework.common.exception.ErrorCode;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.code.CodeUtils;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.object.JavaBeanUtils;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.limiter.dynamic.DynamicRateLimiter;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.framework.velocity.VelocityHelper;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.order.convert.order.OrderConvert;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dto.*;
import cn.iocoder.yudao.module.order.enums.OrderApprovalTypeEnum;
import cn.iocoder.yudao.module.order.enums.OrderTempLateEnum;
import cn.iocoder.yudao.module.order.enums.TransportTypeShortEnum;
import cn.iocoder.yudao.module.order.service.approval.OrderApprovalService;
import cn.iocoder.yudao.module.order.service.order.OrderBusinessService;
import cn.iocoder.yudao.module.order.service.order.OrderItemService;
import cn.iocoder.yudao.module.order.service.order.OrderQueryService;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import cn.iocoder.yudao.module.order.service.orderWarehouseApproval.OrderWarehouseApprovalService;
import cn.iocoder.yudao.module.order.vo.approval.OrderSpecialApplyVO;
import cn.iocoder.yudao.module.order.vo.approval.OrderSpecialBatchApplyVO;
import cn.iocoder.yudao.module.order.vo.order.*;
import cn.iocoder.yudao.module.order.vo.orderItem.ItemPackingListVO;
import cn.iocoder.yudao.module.order.vo.orderItem.OrderItemBackVO;
import cn.iocoder.yudao.module.order.vo.orderWarehouseAdjust.OrderWarehouseAdjustBatchApplyBackVO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductDO;
import cn.iocoder.yudao.module.product.service.product.ProductService;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import cn.iocoder.yudao.module.system.api.permission.RoleApi;
import cn.iocoder.yudao.module.system.api.permission.dto.RoleRespDTO;
import cn.iocoder.yudao.module.system.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.system.enums.permission.DataScopeEnum;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.pdf.PdfWriter;
import com.itextpdf.tool.xml.XMLWorkerHelper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.*;
import java.math.BigDecimal;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.text.MessageFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.enums.GlobalErrorCodeConstants.UNAUTHORIZED;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.ROLE_NOT_EXISTS;

@Validated
@RestController
@Api(tags = "管理后台 - 订单管理")
@RequestMapping("/ecw/order")
@Slf4j
public class OrderController {

    @Autowired
    private OrderService orderService;
    @Autowired
    private OrderBusinessService orderBusinessService;
    @Autowired
    private OrderQueryService orderQueryService;
    @Autowired
    private OrderApprovalService orderApprovalService;
    @Autowired
    private OrderWarehouseApprovalService orderWarehouseApprovalService;
    @Autowired
    private RoleApi roleApi;
    @Autowired
    private OrderItemService orderItemService;
    @Autowired
    private ProductService productService;
    @Autowired
    private DictDataApi dictDataApi;
    @Autowired
    private FileService fileService;

    @Value("${ue.templates-url}")
    private String path;

    @Value("${ue.temp-dir}")
    private String tempDir;

    @Resource
    private RedisHelper redisHelper;

    @Resource
    private ApplicationContext applicationContext;

    /**
     * 订单修改操作的防重复提交缓存key
     */
    private static final String ORDER_UPDATE_KEY = "jiedao:order:update:{0}";

    @PostMapping("/create")
    @ApiOperation("创建订单")
    @DynamicRateLimiter(base = "#Headers['Authorization']", permits = 1)
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:order:create')")
    public CommonResult<Long> createOrder(
            @Valid @RequestBody OrderCreateReqVO createReqVO) {
        if (null == createReqVO.getStatus()) {
            createReqVO.setStatus(0);
        }
        if (Objects.equals(TransportTypeShortEnum.AIR_FREIGHT_LINE.getValue(),
                createReqVO.getTransportId())
                || Objects.equals(TransportTypeShortEnum.AIR_SEA_COMBINED_TRANSPORT.getValue(), createReqVO.getTransportId())) {
            if (Objects.isNull(createReqVO.getChannelId()) ||
                    0 == createReqVO.getChannelId()) {
                //空运渠道不能为空
                throw exception(ORDER_LINE_CHANNEL_NOT_NULL);
            }
            if (StringUtils.isNotEmpty(createReqVO.getNumber())
                    && createReqVO.getNumber().length() > 2048) {
                throw exception(ORDER_EXPRESS_NUMBER_TOO_LONG);
            }
        }

        // member_user 注册会员用户
        Long orderId = orderService.createOrder(createReqVO,
                null).getOrderId();
        return success(orderId);
    }


    @PostMapping("/copy/{copyOrderId}")
    @ApiOperation("复制订单")
    @DynamicRateLimiter(base = "#Headers['Authorization']", permits = 1)
    @Idempotent(timeout = 5)
    public CommonResult<Long> copyOrder(@PathVariable Long copyOrderId) {
        Long orderId = orderService.copyOrder(copyOrderId, null);
        return success(orderId);
    }


    @PutMapping("/update")
    @ApiOperation("更新订单")
    @Idempotent(timeout = 5)
    @Deprecated
//    @PreAuthorize("@ss.hasPermission('ecw:order:update')")
    public CommonResult<Boolean> updateOrder(@Valid @RequestBody OrderUpdateReqVO updateReqVO) {
        if (Objects.equals(TransportTypeShortEnum.AIR_FREIGHT_LINE.getValue(), updateReqVO.getTransportId())
                || Objects.equals(TransportTypeShortEnum.AIR_SEA_COMBINED_TRANSPORT.getValue(), updateReqVO.getTransportId())) {
            if (Objects.isNull(updateReqVO.getChannelId()) || 0 == updateReqVO.getChannelId()) {
                throw exception(ORDER_LINE_CHANNEL_NOT_NULL);
            }
            if (StringUtils.isNotEmpty(updateReqVO.getNumber()) && updateReqVO.getNumber().length() > 2048) {
                throw exception(ORDER_EXPRESS_NUMBER_TOO_LONG);
            }
        }
        String redisKey = MessageFormat.format(ORDER_UPDATE_KEY, updateReqVO.getOrderId().toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1) {
            return error(ORDER_UPDATE_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        orderService.updateOrder(updateReqVO, null);
        redisHelper.delete(redisKey);

        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除订单")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:order:delete')")
    public CommonResult<Boolean> deleteOrder(@RequestParam("orderId") Long orderId) {
        String redisKey = MessageFormat.format(ORDER_UPDATE_KEY, orderId.toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1) {
            return error(ORDER_UPDATE_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        orderService.deleteOrder(orderId);
        redisHelper.delete(redisKey);
        return success(true);
    }

    @PutMapping("/recovery/{orderId}")
    @ApiOperation("恢复订单")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:order:update')")
    public CommonResult<Boolean> recoveryOrder(@PathVariable("orderId") Long orderId) {
        String redisKey = MessageFormat.format(ORDER_UPDATE_KEY, orderId.toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1) {
            return error(ORDER_UPDATE_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        orderService.recoveryOrder(orderId);
        redisHelper.delete(redisKey);
        return success(true);
    }

    @PutMapping("/cancel/{orderId}")
    @ApiOperation("取消订单")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:order:update')")
    public CommonResult<Boolean> cancelOrder(@PathVariable("orderId") Long orderId) {
        String redisKey = MessageFormat.format(ORDER_UPDATE_KEY, orderId.toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1) {
            return error(ORDER_UPDATE_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        orderService.cancelOrder(orderId);
        redisHelper.delete(redisKey);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得编辑详情订单")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:order:query')")
    public CommonResult<OrderBackVO> getOrder(@RequestParam("orderId") Long orderId) {
        OrderBackVO orderBackVO = orderQueryService.getOrder(orderId);
        return success(orderBackVO);
    }

    @GetMapping("/getParentOrder")
    @ApiOperation("获得母订单详情订单")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<OrderBackVO> getParentOrder(@RequestParam("orderId") Long orderId) {
        OrderBackVO orderBackVO = orderQueryService.getParentOrder(orderId);
        return success(orderBackVO);
    }


    @PutMapping("/update/apply")
    @ApiOperation("(新修改订单)订单修改审批申请")
//    @PreAuthorize("@ss.hasPermission('ecw:order:update')")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> orderUpdateApply(@Valid @RequestBody OrderUpdateReqVO updateReqVO) {
        if (Objects.equals(TransportTypeShortEnum.AIR_FREIGHT_LINE.getValue(), updateReqVO.getTransportId())
                || Objects.equals(TransportTypeShortEnum.AIR_SEA_COMBINED_TRANSPORT.getValue(), updateReqVO.getTransportId())) {
            if (Objects.isNull(updateReqVO.getChannelId()) || 0 == updateReqVO.getChannelId()) {
                throw exception(ORDER_LINE_CHANNEL_NOT_NULL);
            }
        }
        String redisKey = MessageFormat.format(ORDER_UPDATE_KEY, updateReqVO.getOrderId().toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1) {
            return error(ORDER_UPDATE_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        orderService.updateOrderApply(updateReqVO, null);
        redisHelper.delete(redisKey);
        return success(true);
    }

    @PutMapping("/shipment/update/apply")
    @ApiOperation("(新修改订单)订单修改审批申请")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> shipmentOrderApplyUpdate(@Valid @RequestBody OrderUpdateReqVO updateReqVO) {
        if (Objects.equals(TransportTypeShortEnum.AIR_FREIGHT_LINE.getValue(), updateReqVO.getTransportId())
                || Objects.equals(TransportTypeShortEnum.AIR_SEA_COMBINED_TRANSPORT.getValue(), updateReqVO.getTransportId())) {
            if (Objects.isNull(updateReqVO.getChannelId()) || 0 == updateReqVO.getChannelId()) {
                throw exception(ORDER_LINE_CHANNEL_NOT_NULL);
            }
        }
        String redisKey = MessageFormat.format(ORDER_UPDATE_KEY, updateReqVO.getOrderId().toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1) {
            return error(ORDER_UPDATE_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        orderService.shipmentOrderApplyUpdate(updateReqVO, null);
        redisHelper.delete(redisKey);
        return success(true);
    }

    @GetMapping("update/info")
    @ApiOperation("获得订单修改申请详情")
    @ApiImplicitParam(name = "approveId", value = "审核ID", required = true, example = "1024", dataType = "Long")
//    @PreAuthorize("@ss.hasPermission('ecw:order-approval:query')")
    public CommonResult<OrderBackVO> getUpdateOrderApplyInfoByApproveId(@RequestParam("approveId") Long approveId) {
        OrderBackVO orderBackVO = orderQueryService.getUpdateOrderApplyInfo(approveId);
        return success(orderBackVO);
    }

    @GetMapping("update/info/{orderId}")
    @ApiOperation("(新获取订单编辑详情)根据订单ID获得订单修改申请详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId", value = "该值为orderId", required = true, example = "1024", dataType = "Long"),
            @ApiImplicitParam(name = "isResubmit", value = "是否重新提交获取 true/false", example = "false", dataType = "Boolean")
    })
//    @PreAuthorize("@ss.hasPermission('ecw:order-approval:query')")
    public CommonResult<OrderBackVO> getUpdateOrderApplyInfoByOrderId(@PathVariable(value = "orderId") Long orderId,
                                                                      @RequestParam(value = "isResubmit", defaultValue = "false") boolean isResubmit) {
        OrderBackVO orderBackVO = orderQueryService.getUpdateOrderApplyInfo(orderId, isResubmit);
        return success(orderBackVO);
    }

    @GetMapping("/info/{orderId}")
    @ApiOperation("订单详情")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:order:query')")
    public CommonResult<OrderBackInfoDto> info(@PathVariable("orderId") Long orderId) {
        OrderBackInfoDto dto = orderQueryService.info(orderId);
        return success(dto);
    }

    @GetMapping("/list")
    @ApiOperation("获得订单列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
//    @PreAuthorize("@ss.hasPermission('ecw:order:query')")
    public CommonResult<List<OrderBackVO>> getOrderList(@RequestParam("ids") Collection<Long> ids) {
        List<OrderDO> list = orderQueryService.getOrderList(ids);
        return success(OrderConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得订单分页")
//    @PreAuthorize("@ss.hasPermission('ecw:order:query')")
    public CommonResult<PageResult<OrderBackPageVO>> getOrderPage(OrderQueryVO query, PageVO page) {
        query.setUserType(UserTypeEnum.ADMIN.getValue());
        PageResult<OrderBackPageVO> pageResult = orderQueryService.orderPage(query, page);
        return success(pageResult);
    }


    @GetMapping("/customer-order-page")
    @ApiOperation("客户详情订单分页")
//    @PreAuthorize("@ss.hasPermission('ecw:order:query')")
    public CommonResult<PageResult<OrderBackPageVO>> getCustomerOrderPage(OrderQueryVO query, PageVO page) {
        query.setUserType(UserTypeEnum.ADMIN.getValue());
        PageResult<OrderBackPageVO> pageResult = orderQueryService.myOrderPage(query, page);

        //填充报价单号
        List<OrderBackPageVO> list = pageResult.getList();
        if (CollectionUtil.isNotEmpty(list)) {
            for (OrderBackPageVO vo : list) {
                OrderAssociationOfferInfoEvent event = new OrderAssociationOfferInfoEvent(vo.getOrderId(), null, null);
                applicationContext.publishEvent(event);
                vo.setOfferId(event.getOfferId());
                vo.setOfferNo(event.getOfferNo());
            }
        }

        return success(pageResult);
    }

    @GetMapping("/my/page")
    @ApiOperation("获得我的订单分页")
    public CommonResult<PageResult<OrderBackPageVO>> myOrderPage(OrderQueryVO query, PageVO page) {
        Long salesmanId = SecurityFrameworkUtils.getLoginUserId(); // 获取当前登录用户ID
        query.setUserType(UserTypeEnum.ADMIN.getValue());
//        query.setSalesmanId(salesmanId);
        query.setCreator(String.valueOf(salesmanId));
        PageResult<OrderBackPageVO> pageResult =
                orderQueryService.myOrderPage(query, page);
        return success(pageResult);
    }


    @GetMapping("/dept-order-page")
    @ApiOperation("部门订单分页")
//    @PreAuthorize("@ss.hasPermission('ecw:order:query')")
    public CommonResult<PageResult<OrderBackPageVO>> getDeptOrderPage(OrderQueryVO query, PageVO page) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        query.setUserType(UserTypeEnum.ADMIN.getValue());
        query.setDeptId(Objects.isNull(loginUser) ? 0L : loginUser.getDeptId());
        PageResult<OrderBackPageVO> pageResult = orderQueryService.deptOrderPage(query, page);
        return success(pageResult);
    }

    @GetMapping("/data-scope-order-page")
    @ApiOperation("数据权限订单分页")
    public CommonResult<PageResult<OrderBackPageVO>> getDataScopeOrderPage(OrderQueryVO query, PageVO page) {
        if (Objects.isNull(query)) {
            query = new OrderQueryVO();
        }
        query.setUserType(UserTypeEnum.ADMIN.getValue());
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        if (Objects.isNull(loginUser)) {
            return error(ErrorCodeConstants.USER_NOT_EXISTS);
        }
        List<RoleRespDTO> roleRespDTOS = roleApi.getRoles(loginUser.getRoleIds());
        if (Objects.isNull(roleRespDTOS)) {
            return error(ROLE_NOT_EXISTS);
        }
        addDataScopeQuery(query, loginUser, roleRespDTOS);
        PageResult<OrderBackPageVO> pageResult = orderQueryService.orderPage(query, page);
        return success(pageResult);
    }


    private static void addDataScopeQuery(OrderQueryVO query,
                                          LoginUser loginUser,
                                          List<RoleRespDTO> roleRespDTOS) {
        List<Long> deptIdList = new ArrayList<>();
        boolean isAll = Boolean.FALSE;
        for (RoleRespDTO roleRespDTO : roleRespDTOS) {
            DataScopeEnum dataScopeEnum = DataScopeEnum.valueOf(roleRespDTO.getDataScope());
            switch (dataScopeEnum) {
                case ALL:
                    // 全部数据权限，不限制
                    isAll = Boolean.TRUE;
                    break;
                case DEPT_CUSTOM:
                    // 指定部门数据权限
                    if (CollectionUtil.isNotEmpty(roleRespDTO.getDataScopeDeptIds())) {
                        deptIdList.addAll(roleRespDTO.getDataScopeDeptIds());
                    }
                    break;
                case DEPT_ONLY:
                    // 部门数据权限
                    if (Objects.nonNull(loginUser.getDeptId())) {
                        deptIdList.add(loginUser.getDeptId());
                    }
                    break;
                case DEPT_AND_CHILD:
                    // 部门及以下数据权限
                    deptIdList.add(loginUser.getDeptId());
                    break;
            }
        }
        if (!isAll) {
            // 非全部数据权限时查询限制条件
            if (CollectionUtil.isEmpty(deptIdList) || deptIdList.size() == 0) {
                // 没有部门限制条件时，只查询当前用户自己的
                query.setCreator(String.valueOf(loginUser.getId()));
            } else {
                // 查询限定部门的数据
                query.setDeptIdList(deptIdList);
            }
        }
    }

    @PutMapping("/special/apply")
    @ApiOperation("特价申请")
    @Idempotent(timeout = 10)
//    @PreAuthorize("@ss.hasPermission('ecw:order:update')")
    public CommonResult<Boolean> specialApplyOrder(@Valid @RequestBody OrderSpecialApplyVO specialApplyVO) {
        orderService.specialApplyOrder(specialApplyVO);
        return success(true);
    }

    @PutMapping("/special/apply/batch")
    @ApiOperation("特价申请-批量特价")
    @Idempotent(timeout = 10)
//    @PreAuthorize("@ss.hasPermission('ecw:order:update')")
    public CommonResult<Boolean> specialBatchApplyOrder(@Valid @RequestBody OrderSpecialBatchApplyVO orderSpecialBatchApplyVO) {
        orderService.specialBatchApplyOrder(orderSpecialBatchApplyVO);
        return success(true);
    }

    @GetMapping("special/info")
    @ApiOperation("获得订单特价申请详情")
    @ApiImplicitParam(name = "approveId", value = "审核ID", required = true, example = "1024", dataType = "Long")
//    @PreAuthorize("@ss.hasPermission('ecw:order-approval:query')")
    public CommonResult<OrderSpecialApplyVO> getOrderApproval(@RequestParam("approveId") Long approveId) {
        OrderSpecialApplyVO applyVO = orderQueryService.approvalOrderSpecial(approveId);
        return success(applyVO);
    }

    @GetMapping("special/batch/info")
    @ApiOperation("获得订单批量特价申请详情-批量特价")
    @ApiImplicitParam(name = "approveId", value = "审核ID", required = true, example = "1024", dataType = "Long")
//    @PreAuthorize("@ss.hasPermission('ecw:order-approval:query')")
    public CommonResult<OrderSpecialBatchApplyVO> getOrderBatchApplyApproval(@RequestParam("approveId") Long approveId) {
        OrderSpecialBatchApplyVO applyVO = orderQueryService.approvalOrderSpecialBatch(approveId);
        return success(applyVO);
    }

    @GetMapping("special/info/{orderId}/{type}")
    @ApiOperation("根据订单项商品ID与审批类型获得订单商品的特价申请详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId", value = "该值为orderId", required = true, example = "1024", dataType = "Long"),
            @ApiImplicitParam(name = "type", value = "申请类型，详见字典order_approval_type", required = true, example = "1024", dataType = "Integer"),
            @ApiImplicitParam(name = "orderItemId", value = "订单项商品orderItemId", example = "1024", dataType = "Long"),
            @ApiImplicitParam(name = "isResubmit", value = "是否重新提交获取 true/false", example = "false", dataType = "Boolean")
    })
//    @PreAuthorize("@ss.hasPermission('ecw:order-approval:query')")
    public CommonResult<OrderSpecialApplyVO> getApprovalOrderSpecialByOrderItemId(@PathVariable(value = "orderId") Long orderId,
                                                                                  @PathVariable(value = "type") Integer type,
                                                                                  @RequestParam(value = "orderItemId", required = false) Long orderItemId,
                                                                                  @RequestParam(value = "isResubmit", defaultValue = "false") boolean isResubmit) {
        OrderSpecialApplyVO applyVO = orderQueryService.approvalOrderSpecialByOrderItemId(orderId, orderItemId, type, isResubmit);
        return success(applyVO);
    }

    @GetMapping("special/info/batch/{orderId}/{type}")
    @ApiOperation("根据订单项商品ID与审批类型获得订单商品的特价申请详情-批量特价")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId", value = "该值为orderId", required = true, example = "1024", dataType = "Long"),
            @ApiImplicitParam(name = "type", value = "申请类型，详见字典order_approval_type", required = true, example = "1024", dataType = "Integer"),
            @ApiImplicitParam(name = "isResubmit", value = "是否重新提交获取 true/false", example = "false", dataType = "Boolean")
    })
//    @PreAuthorize("@ss.hasPermission('ecw:order-approval:query')")
    public CommonResult<OrderSpecialBatchApplyVO> getApprovalOrderSpecialByOrderId(@PathVariable(value = "orderId") Long orderId,
                                                                                   @PathVariable(value = "type") Integer type,
                                                                                   @RequestParam(value = "isResubmit", defaultValue = "false") boolean isResubmit) {
        OrderSpecialBatchApplyVO applyVO = orderQueryService.approvalOrderSpecialByOrderId(orderId, type, isResubmit);
        return success(applyVO);
    }

    @GetMapping("cancel/approval/{orderApprovalId}")
    @ApiOperation("根据审批id取消审批")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderApprovalId", value = "审批ID", required = true, example = "1024", dataType = "Long"),
            @ApiImplicitParam(name = "reason", value = "取消原因", required = true, example = "1024", dataType = "String")
    })
//    @PreAuthorize("@ss.hasPermission('ecw:order-approval:update')")
    public CommonResult<Boolean> cancelOrderSpecialApproval(@PathVariable(value = "orderApprovalId") Long orderApprovalId,
                                                            @RequestParam(value = "reason") String reason) {
        orderService.cancelOrderSpecialApproval(orderApprovalId, reason);
        return success(Boolean.TRUE);
    }


    @GetMapping("/export-excel")
    @ApiOperation("导出订单 Excel")
    @OperateLog(type = EXPORT)
    public void exportOrderExcel(@Valid OrderQueryVO query,
                                 HttpServletResponse response) throws IOException {
        query.setUserType(UserTypeEnum.ADMIN.getValue());
        List<OrderExportBackDTO> list = orderQueryService.getExportOrderList(query);
        // 导出 Excel
        ExcelUtils.write(response, "订单.xls", "sheet1", OrderExportBackDTO.class, list);
    }

    @GetMapping("/preloadPage")
    @ApiOperation("获得订单预装分页")
    public CommonResult<PageResult<OrderPreloadVO>> getOrderPreloadPage(OrderPreloadReq query) {
        PageResult<OrderPreloadVO> pageResult = orderQueryService.orderPreloadPage(query);
        return success(pageResult);
    }


    @GetMapping("/billLading/template")
    @ApiOperation("获取提单pdf")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "billLadingNo", value = "提单编号", required = true, example = "TD20220717", dataTypeClass = String.class),
            @ApiImplicitParam(name = "type", value = "类型:1 海运 2 海空联运 3 空运", required = true, example = "1", dataTypeClass = Integer.class)
    })
    @OperateLog(type = EXPORT)
    @Deprecated
    public CommonResult<String> getBillLadingTemplatePDF(String billLadingNo, Integer type, HttpServletResponse response) throws IOException, DocumentException {
        // 根据类型不同取不同的模板
        OrderTempLateEnum anEnum = OrderTempLateEnum.getEnumByCode(type);
        if (Objects.isNull(anEnum)) {
            return error(new ErrorCode(100000, "类型不存在"));
        }
        String templateName = I18nMessage.getLang() == 0 ? anEnum.getNameZh() : anEnum.getNameEn();
        //设置打印后pdf的格式
        Document document = new Document(PageSize.A4, 60.5f, 60.5f, 33f, 33f);

        PdfWriter pdfWriter = PdfWriter.getInstance(document, response.getOutputStream());// outputStream可以自己指定位置或者是HttpServletResponse得到的输出流
        pdfWriter.setTagged();
        document.open();
        //取得velocity的上下文
        //往vm中写入信息
        //写入自己要插入的信息，
        // TODO 根据提单查询 订单详情
        Map<String, Object> param = new HashMap<>();
        param.put("billLadingNo", "TD20220719E");
        param.put("orderNo", "CNG20220701N");
        param.put("consigneeName", "张三");
        param.put("plNo", "CNG20220701N-01");
        param.put("consigneePhone", "+13289227288");
        param.put("consignorPhone", "+8615677282285");
        param.put("consignorName", "李四");
        param.put("deliveryDate", "2022-07-01");

        // TODO 测试使用指定的测试数据
        List<OrderItemDO> orderItemDOList = orderItemService.list(new LambdaQueryWrapper<OrderItemDO>()
                .eq(OrderItemDO::getOrderNo, "NSA2200001L"));
        if (CollectionUtil.isEmpty(orderItemDOList)) {
            return error(1000000, "该提单没有商品信息");
        }
        List<Map<String, Object>> mapList = JavaBeanUtils.objectsToMaps(orderItemDOList);

        param.put("totalAmount", "66668.99");
        //TODO 读取源文件,未避免服务器上文件路径读取问题,这里直接获取流文件,放入新创建的临时文件中进行处理
        InputStream inputStream = getClass().getClassLoader().getResourceAsStream(path + "/" + templateName + ".html");
        if (inputStream == null) {
            return error(1000000, "未找到模板文件");
        }
        File file = new File(tempDir + "/" + "copy_" + templateName + ".html");
        FileUtils.copyToFile(inputStream, file);
        Writer writer = VelocityHelper.getWriterByTemplate(file, null, param, mapList);
        //输出转输入流
        InputStream in = new ByteArrayInputStream(writer.toString().getBytes(Charset.defaultCharset()));
        //去找velocity并交给itext生成pdf
        XMLWorkerHelper.getInstance().parseXHtml(pdfWriter, document, in, StandardCharsets.UTF_8);
        FileUtils.delete(file);// TODO 删除临时文件
        inputStream.close(); // 关闭文件流
        in.close();
        document.close();

        return success("");
    }


    @GetMapping("/batch-adjust-apply-page")
    @ApiOperation("获得订单批量调仓申请页面分页")
    public CommonResult<PageResult<OrderWarehouseAdjustBatchApplyBackVO>> getOrderWarehouseAdjustBatchApplyPage(OrderQueryVO query, PageVO page) {
        PageResult<OrderWarehouseAdjustBatchApplyBackVO> pageResult = orderQueryService.orderWarehouseAdjustBatchApplyPage(query, page);
        return success(pageResult);
    }


    @GetMapping("/courier/number")
    @ApiOperation("生成快递单号")
    public CommonResult<String> getCourierNumber() {
        // TODO 业务待完善
        return success(CodeUtils.getCourierNumber());
    }


    @PostMapping("/import")
    @ApiOperation("导入装箱单")
    @ApiImplicitParam(name = "file", value = "Excel 文件", required = true, dataTypeClass = MultipartFile.class)
    public CommonResult<PackingListDto> importExcel(@RequestParam("file") MultipartFile file) throws Exception {
        String fix = FileNameUtil.getSuffix(file.getOriginalFilename());
        if (!Objects.equals(fix, "xls") && !Objects.equals(fix, "xlsx")) {
            return error(FILE_FORMAT_IS_EXCEL);
        }
        List<ItemPackingListVO> list = ExcelUtils.read(file, ItemPackingListVO.class);
        if (CollectionUtil.isNotEmpty(list)) {
            for (ItemPackingListVO itemPackingListVO : list) {
                itemPackingListVO.setBrandType(Objects.equals("y", itemPackingListVO.getBrandTitle()) ? 1 : 0);
                ProductDO productDO = productService.findProdByTitle(itemPackingListVO.getProdTitleZh(), itemPackingListVO.getProdTitleEn());
                if (Objects.nonNull(productDO)) {
                    itemPackingListVO.setProdId(productDO.getId());
                }
            }
            String path = DateUtils.getDir() + IdUtil.objectId() + (StrUtil.isBlank(fix) ? StrUtil.EMPTY : StrUtil.DOT + fix);
            String url = fileService.createFile(path, file.getBytes());
            PackingListDto dto = new PackingListDto();
            dto.setPackingListUrl(url);
            dto.setList(list.stream().filter(vo -> Objects.nonNull(vo.getProdId())).collect(Collectors.toList()));
            return success(dto);
        }
        return error(PACKING_LIST_NOT_READ_DATA);
    }

    @GetMapping("/export-packing-list")
    @ApiOperation("获取装箱单模板")
    public void exportExcel(HttpServletResponse response) throws IOException {
        // 随机获取数据库10条可用数据
        List<ProductDO> productDOList = productService.randomTemplateProdData(10);
        List<DictDataRespDTO> dictDataRespDTOList = dictDataApi.getDictDatas("product_material");
        Map<String, DictDataRespDTO> dictDataRespDTOMap = dictDataRespDTOList.stream().collect(Collectors.toMap(DictDataRespDTO::getValue, Function.identity(), (key1, key2) -> key2));
        List<ItemPackingListVO> list = new ArrayList<>();
        for (int i = 0; i < productDOList.size(); i++) {
            ProductDO productDO = productDOList.get(i);
            list.add(ItemPackingListVO.builder()
                    .id(i + 1).num(1)
                    .brandTitle(i % 2 == 0 ? "n" : "y").prodTitleZh(productDO.getTitleZh()).prodTitleEn(productDO.getTitleEn())
                    .lessNum(1).quantity(1).volume(BigDecimal.ONE).weight(BigDecimal.ONE)
                    .material(Objects.isNull(dictDataRespDTOMap.get(productDO.getMaterialType())) ? (I18nMessage.getLang() == 0 ? "未知" : "unknown") :
                            (I18nMessage.getLang() == 0 ? dictDataRespDTOMap.get(productDO.getMaterialType()).getLabel() : dictDataRespDTOMap.get(productDO.getMaterialType()).getLabelEn()))
                    .build());
        }
        ExcelUtils.write(response, I18nMessage.getLang() == 0 ? "装箱单模板.xls" : "packing-list-template.xls", "数据", ItemPackingListVO.class, list);
    }


    @GetMapping("/orderNoSearch")
    @ApiOperation("订单号检索功能")
    public CommonResult<PageResult<SearchBackDto>> orderNoSearch(@RequestParam("key") String key, PageParam param) {
        return success(orderQueryService.orderNoSearch(key, param));
    }

    @GetMapping("/billNoSearch")
    @ApiOperation("提单号检索功能")
    public CommonResult<PageResult<SearchBackDto>> billNoSearch(@RequestParam("key") String key, PageParam param) {
        return success(orderQueryService.billNoSearch(key, param));
    }


    @PutMapping("/refresh-customer-performance")
    @ApiOperation(value = "更新订单发货人与收货人信息，且更新业绩归属")
    @ApiImplicitParam(value = "订单编号", name = "orderNumbers", required = true, example = "示例：NG2300208L,NG2300212L,NG2300209L多个订单编号逗号连接", dataTypeClass = String.class)
    public CommonResult<Boolean> updateOrderCustomer(@RequestParam(value = "orderNumbers") String orderNumbers) {
        List<String> list = new ArrayList<>();
        if (StringUtils.isNotBlank(orderNumbers)) {
            list.addAll(Arrays.asList(orderNumbers.split(StrUtil.COMMA)));
        }
        orderBusinessService.updateOrderCustomer(list);
        return success(true);
    }

    @PutMapping("/refresh-order-price")
    @ApiOperation(value = "重置订单价格")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "订单编号", name = "orderNumbers", required = false, example = "示例：NG2300208L,NG2300212L,NG2300209L多个订单编号逗号连接", dataTypeClass = String.class),
            @ApiImplicitParam(value = "是否全部重置", name = "isAllPrice", required = false, example = "是否全部重置-true/false", dataTypeClass = Boolean.class),
            @ApiImplicitParam(value = "是否重新定价", name = "isResetPrice", required = true, example = "是否重新定价-true/false", dataTypeClass = Boolean.class)
    })
    public CommonResult<Boolean> updateOrderPrice(@RequestParam(value = "orderNumbers", required = false) String orderNumbers,
                                                  @RequestParam(value = "isAllPrice", defaultValue = "false") Boolean isAllPrice,
                                                  @RequestParam(value = "isResetPrice") Boolean isResetPrice) {
        List<String> list = new ArrayList<>();
        if (StringUtils.isBlank(orderNumbers) && (Objects.isNull(isAllPrice) || !isAllPrice)) {
            return error(new ErrorCode(11110000, "请确认是否全部重置"));
        }
        if (StringUtils.isNotBlank(orderNumbers)) {
            list.addAll(Arrays.asList(orderNumbers.split(StrUtil.COMMA)));
        }
        orderService.updateOrderPrice(list, isResetPrice);
        return success(true);
    }


    @PutMapping("/create-order-receivable")
    @ApiOperation(value = "手动创建订单的运费与清关费应收单", notes = "当运费与清关费应收单因为bug而没生成时，才能执行")
    @ApiImplicitParam(value = "订单编号", name = "orderNo", required = true, example = "示例：NG2300208L", dataTypeClass = String.class)
    public CommonResult<Boolean> createOrderReceivable(@RequestParam(value = "orderNo") String orderNo) {
        orderBusinessService.updateReceivable(orderNo);
        return success(true);
    }

    @PutMapping("/update/guanlianOrderStatus")
    @ApiOperation(value = "更新关联订单状态")
    @ApiImplicitParam(value = "订单编号", name = "orderNumbers", example = "示例：NG2300208L,NG2300212L,NG2300209L多个订单编号逗号连接", dataTypeClass = String.class)
    public void updateGuanlianOrderStatus(@RequestParam(value = "orderNumbers", required = false) String orderNumbers) {
        List<String> list = null;
        if (StringUtils.isNotBlank(orderNumbers)) {
            list = new ArrayList<>(Arrays.asList(orderNumbers.split(StrUtil.COMMA)));
        }
        orderService.updateGuanlianOrderStatus(list);
    }

    @PutMapping("/update/orderOrgWeightUnitAndOrgVolumeUnit")
    @ApiOperation(value = "更新订单的重泡货原始标准")
    @ApiImplicitParam(value = "订单编号", name = "orderNumbers", example = "示例：NG2300208L,NG2300212L,NG2300209L多个订单编号逗号连接", dataTypeClass = String.class)
    public void updateOrderWeightUnitAndVolumeUnit(@RequestParam(value = "orderNumbers", required = false) String orderNumbers) {
        List<String> list = null;
        if (StringUtils.isNotBlank(orderNumbers)) {
            list = new ArrayList<>(Arrays.asList(orderNumbers.split(StrUtil.COMMA)));
        }
        orderBusinessService.updateOrderOrgWeightUnitAndOrgVolumeUnit(list);
    }

    @GetMapping("/dept/statistics")
    @ApiOperation("参数查询获得部门订单统计")
    public CommonResult<StatisticsOrderVO> statisticsDeptOrder(OrderQueryVO query) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser(); // 获取当前登录用户ID
        query.setUserType(UserTypeEnum.ADMIN.getValue());
//        query.setSalesmanId(salesmanId);
        query.setDeptId(Objects.isNull(loginUser) || Objects.isNull(loginUser.getDeptId()) ? 0L : loginUser.getDeptId());
        return success(orderQueryService.statisticsMyOrder(query));
    }

    @GetMapping("/data/scope/statistics")
    @ApiOperation("参数查询获得数据权限范围订单统计")
    public CommonResult<StatisticsOrderVO> statisticsDataScopeOrder(OrderQueryVO query) {
        if (Objects.isNull(query)) {
            query = new OrderQueryVO();
        }
        query.setUserType(UserTypeEnum.ADMIN.getValue());
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        if (Objects.isNull(loginUser)) {
            return error(ErrorCodeConstants.USER_NOT_EXISTS);
        }
        List<RoleRespDTO> roleRespDTOS = roleApi.getRoles(loginUser.getRoleIds());
        if (Objects.isNull(roleRespDTOS)) {
            return error(ROLE_NOT_EXISTS);
        }
        addDataScopeQuery(query, loginUser, roleRespDTOS);
        query.setUserType(UserTypeEnum.ADMIN.getValue());
        return success(orderQueryService.statisticsOrder(query));
    }

    @GetMapping("/my/statistics")
    @ApiOperation("参数查询获得我的订单统计")
    public CommonResult<StatisticsOrderVO> statisticsMyOrder(OrderQueryVO query) {
        Long salesmanId = SecurityFrameworkUtils.getLoginUserId(); // 获取当前登录用户ID
        query.setUserType(UserTypeEnum.ADMIN.getValue());
//        query.setSalesmanId(salesmanId);
        query.setCreator(String.valueOf(salesmanId));
        return success(orderQueryService.statisticsMyOrder(query));
    }


    @GetMapping("/statistics")
    @ApiOperation("参数查询订单管理统计")
    public CommonResult<StatisticsOrderVO> statisticsOrder(OrderQueryVO query) {
        return success(orderQueryService.statisticsMyOrder(query));
    }


    @GetMapping("/certificateSearch")
    @ApiOperation("发货人证件查询")
    public CommonResult<OrderCertificateVO> certificateSearch(@RequestParam("key") String key) {
        return success(orderQueryService.certificateSearch(key));
    }

    @GetMapping("/getDestCountryByOrderId")
    @ApiOperation("根据订单ID查询目的国")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<String> getDestCountryByOrderId(Long orderId) {
        return success(orderQueryService.getDestCountryByOrderId(orderId));
    }


    @GetMapping("/order-item-info-when-warehouse-in-deleted/{orderId}")
    @ApiOperation("退仓品名列表")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<OrderItemBackVO>> getOrderItemInfoListWhenWarehouseInDeleted(@PathVariable("orderId") Long orderId) {
        List<OrderItemBackVO> orderItemBackVOList = orderQueryService.getOrderItemInfoListWhenWarehouseInDeleted(orderId);
        return success(orderItemBackVOList);
    }


    @GetMapping("/order-packed-page-statistics/{orderId}")
    @ApiOperation("打包页面订单数据统计")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<OrderPackedPageStatisticsVO> orderPackedPageStatistics(@PathVariable("orderId") Long orderId) {
        OrderPackedPageStatisticsVO statisticsVO = orderQueryService.orderPackedPageStatistics(orderId);
        return success(statisticsVO);
    }


    @GetMapping("/approvalTypeCheck")
    @ApiOperation("校验订单审批是否存在互斥类型：是 或 否  ")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId", value = "订单id", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "orderItemIdList", value = "订单项id列表(针对品名的审批需要传此参数)", required = false, example = "1024", dataTypeClass = List.class),
            @ApiImplicitParam(name = "approvalType", value = "审批类型：见字典表", required = true, example = "1024", dataTypeClass = Integer.class)
    })
    public CommonResult<Boolean> approvalTypeCheck(Long orderId, List<Long> orderItemIdList, Integer approvalType) {
        OrderApprovalTypeEnum approvalTypeEnum = OrderApprovalTypeEnum.valueOf(approvalType);
        if (Objects.isNull(approvalTypeEnum)){
            // 没有配置相关审批类型则不做校验
            return success(Boolean.FALSE);
        }
        List<Integer> types = approvalTypeEnum.getMutualExclusionValues();
        if (Objects.isNull(types)){
            // 没有配置相关审批类型则不做校验
            return success(Boolean.FALSE);
        }
        // 枚举配置中的互斥类型集合不能添加新值，这里需要重新定义新集合补充当前审批类型进行查询
        List<Integer> typeList = null;
        // 包含-1则表示订单不能有任何审批状态存在,所以不用区分类型，查询所有的进行中审批
        if (!types.contains(-1)){
            typeList = new ArrayList<>(types);
            typeList.add(approvalType);
        }
        // 查询当前订单正在进行的审批
        boolean result = orderApprovalService.countProcessingApproval(orderId, orderItemIdList, typeList);
        // 入仓相关正在进行的审批业务查询
        if (!result){
            List<Integer> warehouseApprovalTypeList = null;
            if (Objects.nonNull(typeList)){
                // 1 入仓修改 2 调仓 3 退仓
                warehouseApprovalTypeList = new ArrayList<>();
                if (typeList.contains(OrderApprovalTypeEnum.WAREHOUSE_UPDATE.getValue())){
                    warehouseApprovalTypeList.add(1);
                }
                if (typeList.contains(OrderApprovalTypeEnum.WAREHOUSE_ADJUST.getValue())){
                    warehouseApprovalTypeList.add(2);
                }
                if (typeList.contains(OrderApprovalTypeEnum.WAREHOUSE_ROLLBACK.getValue())){
                    warehouseApprovalTypeList.add(3);
                }
                if (warehouseApprovalTypeList.size() == 0){
                    return success(result);
                }
            }
            result = orderWarehouseApprovalService.countProcessingApproval(orderId, orderItemIdList, warehouseApprovalTypeList);
        }
        // TODO 当订单互斥所有审批类型时，是否也要对自编号的审批状态进行校验
        return success(result);
    }


}
